﻿using System;
using System.Collections.Generic;

namespace EsempioStrategy
{
    public static class AlignerFactory
    {
        private static readonly Dictionary<string, IAligner> _dictionary;

        static AlignerFactory()
        {
            _dictionary = new Dictionary<string, IAligner>();
            _dictionary.Add("Left", new LeftAligner());
            _dictionary.Add("Right", new RightAligner());
            _dictionary.Add("Center", new CenterAligner());
            _dictionary.Add("Justify", new Justifier());
        }

        public static IAligner GetAligner(string alignerName)
        {
            if (!_dictionary.ContainsKey(alignerName))
                throw new ArgumentException(alignerName);
            return _dictionary[alignerName];
        }

        private abstract class AlignerBase : IAligner
        {
            #region IAligner Members

            public string Format(string text, int width)
            {
                string formattedText = String.Empty;
                string line = String.Empty;
                foreach (string word in text.Split())
                {
                    if (line.Length == 0)
                    {
                        line = word;
                    }
                    else if (line.Length + word.Length + 1 > width)
                    {
                        formattedText += FormatLine(line, width) + '\n';
                        line = word;
                    }
                    else
                    {
                        line += " " + word;
                    }
                }
                formattedText += FormatLine(line, width);
                return formattedText;
            }

            #endregion

            protected abstract string FormatLine(string line, int width);
        }

        private class LeftAligner : AlignerBase
        {
            protected override string FormatLine(string line, int width)
            {
                return line;
            }
        }

        private class RightAligner : AlignerBase
        {
            protected override string FormatLine(string line, int width)
            {
                int offset = width - line.Length;
                if (offset > 0)
                    line = new String(' ', offset) + line;
                return line;
            }
        }

        private class CenterAligner : AlignerBase
        {
            protected override string FormatLine(string line, int width)
            {
                int offset = (width - line.Length) / 2;
                if (offset > 0)
                    line = new String(' ', offset) + line;
                return line;
            }
        }

        private class Justifier : AlignerBase
        {
            protected override string FormatLine(string line, int width)
            {
                int offset = width - line.Length;
                if (offset > 0)
                {
                    string[] word = line.Split();
                    int numWords = word.Length;  //  Numero di parole che compongono la linea
                    for (int numBlanks = 1; offset > 0; numBlanks++)
                    {
                        int startIndex = 0;  //  Posizione della linea in cui inserire un bianco
                        for (int i = 0; i < numWords - 1 && offset > 0; i++)
                        {
                            startIndex += word[i].Length + numBlanks;
                            line = line.Insert(startIndex, " ");
                            offset--;
                            startIndex++;
                        }
                    }
                }
                return line;
            }
        }
    }
}
